@@ -2,7 +2,7 @@
 #
 # SPDX-License-Identifier: BSD-2-Clause
 #
-# Copyright (c) 2018-2020 Gavin D. Howard and contributors.
+# Copyright (c) 2018-2021 Gavin D. Howard and contributors.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -33,6 +33,8 @@ script=$(basename "$script")
 
 . "$scriptdir/functions.sh"
 
+cd "$scriptdir"
+
 usage() {
 
 	if [ $# -gt 0 ]; then
@@ -48,7 +50,7 @@ usage() {
 	printf 'usage:\n'
 	printf '    %s -h\n' "$script"
 	printf '    %s --help\n' "$script"
-	printf '    %s [-a|-bD|-dB|-c] [-EfgGHlMNPT] [-O OPT_LEVEL] [-k KARATSUBA_LEN]\n' "$script"
+	printf '    %s [-a|-bD|-dB|-c] [-CEfgGHlmMNPtTvz] [-O OPT_LEVEL] [-k KARATSUBA_LEN]\n' "$script"
 	printf '    %s \\\n' "$script"
 	printf '       [--library|--bc-only --disable-dc|--dc-only --disable-bc|--coverage]\\\n'
 	printf '       [--force --debug --disable-extra-math --disable-generated-tests]    \\\n'
@@ -73,6 +75,8 @@ usage() {
 	printf '        Generate test coverage code. Requires gcov and regcovr.\n'
 	printf '        It is an error if either "-b" ("-D") or "-d" ("-B") is specified.\n'
 	printf '        Requires a compiler that use gcc-compatible coverage options\n'
+	printf '    -C, --disable-clean\n'
+	printf '        Disable the clean that configure.sh does before configure.\n'
 	printf '    -d, --dc-only\n'
 	printf '        Build dc only. It is an error if "-b", "--bc-only", "-D", or\n'
 	printf '        "--disable-dc" are specified too.\n'
@@ -107,6 +111,8 @@ usage() {
 	printf '        Installs all locales, regardless of how many are on the system. This\n'
 	printf '        option is useful for package maintainers who want to make sure that\n'
 	printf '        a package contains all of the locales that end users might need.\n'
+	printf '    -m, --enable-memcheck\n'
+	printf '        Enable memcheck mode, to ensure no memory leaks. For development only.\n'
 	printf '    -M, --disable-man-pages\n'
 	printf '        Disable installing manpages.\n'
 	printf '    -N, --disable-nls\n'
@@ -119,9 +125,15 @@ usage() {
 	printf '        Disables the prompt in the built bc. The prompt will never show up,\n'
 	printf '        or in other words, it will be permanently disabled and cannot be\n'
 	printf '        enabled.\n'
+	printf '    -t, --enable-test-timing\n'
+	printf '        Enable the timing of tests. This is for development only.\n'
 	printf '    -T, --disable-strip\n'
 	printf '        Disable stripping symbols from the compiled binary or binaries.\n'
 	printf '        Stripping symbols only happens when debug mode is off.\n'
+	printf '    -v, --enable-valgrind\n'
+	printf '        Enable a build appropriate for valgrind. For development only.\n'
+	printf '    -z, --enable-fuzz-mode\n'
+	printf '        Enable fuzzing mode. THIS IS FOR DEVELOPMENT ONLY.\n'
 	printf '    --prefix PREFIX\n'
 	printf '        The prefix to install to. Overrides "$PREFIX" if it exists.\n'
 	printf '        If PREFIX is "/usr", install path will be "/usr/bin".\n'
@@ -241,7 +253,7 @@ replace_ext() {
 	_replace_ext_ext1="$2"
 	_replace_ext_ext2="$3"
 
-	_replace_ext_result=${_replace_ext_file%.$_replace_ext_ext1}.$_replace_ext_ext2
+	_replace_ext_result="${_replace_ext_file%.$_replace_ext_ext1}.$_replace_ext_ext2"
 
 	printf '%s\n' "$_replace_ext_result"
 }
@@ -277,6 +289,23 @@ replace() {
 	substring_replace "$_replace_str" "%%$_replace_needle%%" "$_replace_replacement"
 }
 
+find_src_files() {
+
+	if [ "$#" -ge 1 ] && [ "$1" != "" ]; then
+
+		while [ "$#" -ge 1 ]; do
+			_find_src_files_a="${1## }"
+			shift
+			_find_src_files_args="$_find_src_files_args ! -path src/${_find_src_files_a}"
+		done
+
+	else
+		_find_src_files_args="-print"
+	fi
+
+	printf '%s\n' $(find src/ -depth -name "*.c" $_find_src_files_args)
+}
+
 gen_file_list() {
 
 	if [ "$#" -lt 1 ]; then
@@ -291,15 +320,9 @@ gen_file_list() {
 	cd "$scriptdir"
 
 	if [ "$#" -ge 1 ]; then
-
-		while [ "$#" -ge 1 ]; do
-			a="$1"
-			shift
-			args="$args ! -path src/${a}"
-		done
-
+		_gen_file_list_unneeded="$@"
 	else
-		args="-print"
+		_gen_file_list_unneeded=""
 	fi
 
 	_gen_file_list_needle_src="SRC"
@@ -307,7 +330,7 @@ gen_file_list() {
 	_gen_file_list_needle_gcda="GCDA"
 	_gen_file_list_needle_gcno="GCNO"
 
-	_gen_file_list_replacement=$(find src/ -depth -name "*.c" $args | tr '\n' ' ')
+	_gen_file_list_replacement=$(find_src_files $_gen_file_list_unneeded | tr '\n' ' ')
 	_gen_file_list_contents=$(replace "$_gen_file_list_contents" \
 		"$_gen_file_list_needle_src" "$_gen_file_list_replacement")
 
@@ -328,6 +351,101 @@ gen_file_list() {
 	printf '%s\n' "$_gen_file_list_contents"
 }
 
+gen_tests() {
+
+	_gen_tests_name="$1"
+	shift
+
+	_gen_tests_uname="$1"
+	shift
+
+	_gen_tests_extra_math="$1"
+	shift
+
+	_gen_tests_time_tests="$1"
+	shift
+
+	_gen_tests_extra_required=$(cat tests/extra_required.txt)
+
+	for _gen_tests_t in $(cat "$scriptdir/tests/$_gen_tests_name/all.txt"); do
+
+		if [ "$_gen_tests_extra_math" -eq 0 ]; then
+
+			if [ -z "${_gen_tests_extra_required##*$_gen_tests_t*}" ]; then
+				printf 'test_%s_%s:\n\t@printf "Skipping %s %s\\n"\n\n' \
+					"$_gen_tests_name" "$_gen_tests_t" "$_gen_tests_name" \
+					"$_gen_tests_t" >> "$scriptdir/Makefile"
+				continue
+			fi
+
+		fi
+
+		printf 'test_%s_%s:\n\t@sh tests/test.sh %s %s %s %s %s\n\n' \
+			"$_gen_tests_name" "$_gen_tests_t" "$_gen_tests_name" \
+			"$_gen_tests_t" "$generate_tests" "$time_tests" \
+			"$*" >> "$scriptdir/Makefile"
+
+	done
+}
+
+gen_test_targets() {
+
+	_gen_test_targets_name="$1"
+	shift
+
+	_gen_test_targets_tests=$(cat "$scriptdir/tests/${_gen_test_targets_name}/all.txt")
+
+	for _gen_test_targets_t in $_gen_test_targets_tests; do
+		printf ' test_%s_%s' "$_gen_test_targets_name" "$_gen_test_targets_t"
+	done
+
+	printf '\n'
+}
+
+gen_script_tests() {
+
+	_gen_script_tests_name="$1"
+	shift
+
+	_gen_script_tests_extra_math="$1"
+	shift
+
+	_gen_script_tests_generate="$1"
+	shift
+
+	_gen_script_tests_time="$1"
+	shift
+
+	_gen_script_tests_tests=$(cat "$scriptdir/tests/$_gen_script_tests_name/scripts/all.txt")
+
+	for _gen_script_tests_f in $_gen_script_tests_tests; do
+
+		_gen_script_tests_b=$(basename "$_gen_script_tests_f" ".${_gen_script_tests_name}")
+
+		printf 'test_%s_script_%s:\n\t@sh tests/script.sh %s %s %s 1 %s %s %s\n\n' \
+			"$_gen_script_tests_name" "$_gen_script_tests_b" "$_gen_script_tests_name" \
+			"$_gen_script_tests_f" "$_gen_script_tests_extra_math" "$_gen_script_tests_generate" \
+			"$_gen_script_tests_time" "$*" >> "$scriptdir/Makefile"
+	done
+}
+
+gen_script_test_targets() {
+
+	_gen_script_test_targets_name="$1"
+	shift
+
+	_gen_script_test_targets_tests=$(cat "$scriptdir/tests/$_gen_script_test_targets_name/scripts/all.txt")
+
+	for _gen_script_test_targets_f in $_gen_script_test_targets_tests; do
+		_gen_script_test_targets_b=$(basename "$_gen_script_test_targets_f" \
+			".$_gen_script_test_targets_name")
+		printf ' test_%s_script_%s' "$_gen_script_test_targets_name" \
+			"$_gen_script_test_targets_b"
+	done
+
+	printf '\n'
+}
+
 bc_only=0
 dc_only=0
 coverage=0
@@ -344,14 +462,20 @@ force=0
 strip_bin=1
 all_locales=0
 library=0
+fuzz=0
+time_tests=0
+vg=0
+memcheck=0
+clean=1
 
-while getopts "abBcdDEfgGhHk:lMNO:PST-" opt; do
+while getopts "abBcdDEfgGhHk:lMmNO:PStTvz-" opt; do
 
 	case "$opt" in
 		a) library=1 ;;
 		b) bc_only=1 ;;
 		B) dc_only=1 ;;
 		c) coverage=1 ;;
+		C) clean=0 ;;
 		d) dc_only=1 ;;
 		D) bc_only=1 ;;
 		E) extra_math=0 ;;
@@ -362,11 +486,15 @@ while getopts "abBcdDEfgGhHk:lMNO:PST-" opt; do
 		H) hist=0 ;;
 		k) karatsuba_len="$OPTARG" ;;
 		l) all_locales=1 ;;
+		m) memcheck=1 ;;
 		M) install_manpages=0 ;;
 		N) nls=0 ;;
 		O) optimization="$OPTARG" ;;
 		P) prompt=0 ;;
+		t) time_tests=1 ;;
 		T) strip_bin=0 ;;
+		v) vg=1 ;;
+		z) fuzz=1 ;;
 		-)
 			arg="$1"
 			arg="${arg#--}"
@@ -465,6 +593,7 @@ while getopts "abBcdDEfgGhHk:lMNO:PST-" opt; do
 					shift ;;
 				disable-bc) dc_only=1 ;;
 				disable-dc) bc_only=1 ;;
+				disable-clean) clean=0 ;;
 				disable-extra-math) extra_math=0 ;;
 				disable-generated-tests) generate_tests=0 ;;
 				disable-history) hist=0 ;;
@@ -472,16 +601,24 @@ while getopts "abBcdDEfgGhHk:lMNO:PST-" opt; do
 				disable-nls) nls=0 ;;
 				disable-prompt) prompt=0 ;;
 				disable-strip) strip_bin=0 ;;
+				enable-test-timing) time_tests=1 ;;
+				enable-valgrind) vg=1 ;;
+				enable-fuzz-mode) fuzz=1 ;;
+				enable-memcheck) memcheck=1 ;;
 				install-all-locales) all_locales=1 ;;
 				help* | bc-only* | dc-only* | coverage* | debug*)
 					usage "No arg allowed for --$arg option" ;;
-				disable-bc* | disable-dc* | disable-extra-math*)
+				disable-bc* | disable-dc* | disable-clean*)
+					usage "No arg allowed for --$arg option" ;;
+				disable-extra-math*)
 					usage "No arg allowed for --$arg option" ;;
 				disable-generated-tests* | disable-history*)
 					usage "No arg allowed for --$arg option" ;;
 				disable-man-pages* | disable-nls* | disable-strip*)
 					usage "No arg allowed for --$arg option" ;;
-				install-all-locales*)
+				enable-fuzz-mode* | enable-test-timing* | enable-valgrind*)
+					usage "No arg allowed for --$arg option" ;;
+				enable-memcheck* | install-all-locales*)
 					usage "No arg allowed for --$arg option" ;;
 				'') break ;; # "--" terminates argument processing
 				* ) usage "Invalid option $LONG_OPTARG" ;;
@@ -493,6 +630,12 @@ while getopts "abBcdDEfgGhHk:lMNO:PST-" opt; do
 
 done
 
+if [ "$clean" -ne 0 ]; then
+	if [ -f ./Makefile ]; then
+		make clean_config > /dev/null
+	fi
+fi
+
 if [ "$bc_only" -eq 1 ] && [ "$dc_only" -eq 1 ]; then
 	usage "Can only specify one of -b(-D) or -d(-B)"
 fi
@@ -581,21 +724,19 @@ executable="BC_EXEC"
 tests="test_bc timeconst test_dc"
 
 bc_test="@tests/all.sh bc $extra_math 1 $generate_tests 0 \$(BC_EXEC)"
-bc_time_test="@tests/all.sh bc $extra_math 1 $generate_tests 1 \$(BC_EXEC)"
-
 dc_test="@tests/all.sh dc $extra_math 1 $generate_tests 0 \$(DC_EXEC)"
-dc_time_test="@tests/all.sh dc $extra_math 1 $generate_tests 1 \$(DC_EXEC)"
 
 timeconst="@tests/bc/timeconst.sh tests/bc/scripts/timeconst.bc \$(BC_EXEC)"
 
 # In order to have cleanup at exit, we need to be in
 # debug mode, so don't run valgrind without that.
-if [ "$debug" -ne 0 ]; then
-	vg_bc_test="@tests/all.sh bc $extra_math 1 $generate_tests 0 valgrind \$(VALGRIND_ARGS) \$(BC_EXEC)"
-	vg_dc_test="@tests/all.sh dc $extra_math 1 $generate_tests 0 valgrind \$(VALGRIND_ARGS) \$(DC_EXEC)"
+if [ "$vg" -ne 0 ]; then
+	debug=1
+	bc_test_exec='valgrind $(VALGRIND_ARGS) $(BC_EXEC)'
+	dc_test_exec='valgrind $(VALGRIND_ARGS) $(DC_EXEC)'
 else
-	vg_bc_test="@printf 'Cannot run valgrind without debug flags\\\\n'"
-	vg_dc_test="@printf 'Cannot run valgrind without debug flags\\\\n'"
+	bc_test_exec='$(BC_EXEC)'
+	dc_test_exec='$(DC_EXEC)'
 fi
 
 karatsuba="@printf 'karatsuba cannot be run because one of bc or dc is not built\\\\n'"
@@ -605,7 +746,29 @@ bc_lib="\$(GEN_DIR)/lib.o"
 bc_help="\$(GEN_DIR)/bc_help.o"
 dc_help="\$(GEN_DIR)/dc_help.o"
 
-if [ "$bc_only" -eq 1 ]; then
+default_target_prereqs="\$(BIN) \$(OBJS)"
+default_target_cmd="\$(CC) \$(CFLAGS) \$(OBJS) \$(LDFLAGS) -o \$(EXEC)"
+default_target="\$(DC_EXEC)"
+
+second_target_prereqs=""
+second_target_cmd="$default_target_cmd"
+second_target="\$(BC_EXEC)"
+
+if [ "$library" -ne 0 ]; then
+
+	extra_math=1
+	nls=0
+	hist=0
+	prompt=0
+	bc=1
+	dc=1
+
+	default_target_prereqs="\$(BIN) \$(OBJ)"
+	default_target_cmd="ar -r -cu \$(LIBBC) \$(OBJ)"
+	default_target="\$(LIBBC)"
+	tests="test_library"
+
+elif [ "$bc_only" -eq 1 ]; then
 
 	bc=1
 	dc=0
@@ -615,14 +778,16 @@ if [ "$bc_only" -eq 1 ]; then
 	executables="bc"
 
 	dc_test="@printf 'No dc tests to run\\\\n'"
-	dc_time_test="@printf 'No dc tests to run\\\\n'"
-	vg_dc_test="@printf 'No dc tests to run\\\\n'"
 
 	install_prereqs=" install_execs"
 	install_man_prereqs=" install_bc_manpage"
 	uninstall_prereqs=" uninstall_bc"
 	uninstall_man_prereqs=" uninstall_bc_manpage"
 
+	default_target="\$(BC_EXEC)"
+	second_target="\$(DC_EXEC)"
+	tests="test_bc timeconst"
+
 elif [ "$dc_only" -eq 1 ]; then
 
 	bc=0
@@ -637,8 +802,6 @@ elif [ "$dc_only" -eq 1 ]; then
 	executable="DC_EXEC"
 
 	bc_test="@printf 'No bc tests to run\\\\n'"
-	bc_time_test="@printf 'No bc tests to run\\\\n'"
-	vg_bc_test="@printf 'No bc tests to run\\\\n'"
 
 	timeconst="@printf 'timeconst cannot be run because bc is not built\\\\n'"
 
@@ -647,6 +810,8 @@ elif [ "$dc_only" -eq 1 ]; then
 	uninstall_prereqs=" uninstall_dc"
 	uninstall_man_prereqs=" uninstall_dc_manpage"
 
+	tests="test_dc"
+
 else
 
 	bc=1
@@ -654,8 +819,6 @@ else
 
 	executables="bc and dc"
 
-	link="\$(LINK) \$(BIN) \$(EXEC_PREFIX)\$(DC)"
-
 	karatsuba="@\$(KARATSUBA) 30 0 \$(BC_EXEC)"
 	karatsuba_test="@\$(KARATSUBA) 1 100 \$(BC_EXEC)"
 
@@ -672,6 +835,18 @@ else
 		tests="test_library"
 	fi
 
+	second_target_prereqs="$default_target_prereqs"
+	default_target_prereqs="$second_target"
+	default_target_cmd="\$(LINK) \$(BIN) \$(EXEC_PREFIX)\$(DC)"
+
+fi
+
+if [ "$fuzz" -ne 0 ]; then
+	debug=1
+	hist=0
+	prompt=0
+	nls=0
+	optimization="3"
 fi
 
 if [ "$debug" -eq 1 ]; then
@@ -763,16 +938,6 @@ else
 	uninstall_man_prereqs=""
 fi
 
-if [ "$library" -ne 0 ]; then
-	extra_math=1
-	nls=0
-	hist=0
-	prompt=0
-	ALL_PREREQ="library"
-else
-	ALL_PREREQ="execs"
-fi
-
 if [ "$nls" -ne 0 ]; then
 
 	set +e
@@ -913,13 +1078,21 @@ else
 fi
 
 manpage_args=""
+unneeded=""
+headers="\$(HEADERS)"
 
 if [ "$extra_math" -eq 0 ]; then
 	manpage_args="E"
+	unneeded="$unneeded rand.c"
+else
+	headers="$headers \$(EXTRA_MATH_HEADERS)"
 fi
 
 if [ "$hist" -eq 0 ]; then
 	manpage_args="${manpage_args}H"
+	unneeded="$unneeded history.c"
+else
+	headers="$headers \$(HISTORY_HEADERS)"
 fi
 
 if [ "$nls" -eq 0 ]; then
@@ -930,26 +1103,16 @@ if [ "$prompt" -eq 0 ]; then
 	manpage_args="${manpage_args}P"
 fi
 
-if [ "$manpage_args" = "" ]; then
-	manpage_args="A"
-fi
-
-unneeded=""
-
-if [ "$hist" -eq 0 ]; then
-	unneeded="$unneeded history.c"
-fi
-
 if [ "$bc" -eq 0 ]; then
 	unneeded="$unneeded bc.c bc_lex.c bc_parse.c"
+else
+	headers="$headers \$(BC_HEADERS)"
 fi
 
 if [ "$dc" -eq 0 ]; then
 	unneeded="$unneeded dc.c dc_lex.c dc_parse.c"
-fi
-
-if [ "$extra_math" -eq 0 ]; then
-	unneeded="$unneeded rand.c"
+else
+	headers="$headers \$(DC_HEADERS)"
 fi
 
 if [ "$library" -ne 0 ]; then
@@ -957,10 +1120,24 @@ if [ "$library" -ne 0 ]; then
 	unneeded="$unneeded lang.c lex.c parse.c program.c"
 	unneeded="$unneeded bc.c bc_lex.c bc_parse.c"
 	unneeded="$unneeded dc.c dc_lex.c dc_parse.c"
+	headers="$headers \$(LIBRARY_HEADERS)"
 else
 	unneeded="$unneeded library.c"
 fi
 
+if [ "$manpage_args" = "" ]; then
+	manpage_args="A"
+fi
+
+if [ "$vg" -ne 0 ]; then
+	memcheck=1
+fi
+
+bc_tests=$(gen_test_targets bc)
+bc_script_tests=$(gen_script_test_targets bc)
+dc_tests=$(gen_test_targets dc)
+dc_script_tests=$(gen_script_test_targets dc)
+
 # Print out the values; this is for debugging.
 if [ "$bc" -ne 0 ]; then
 	printf 'Building bc\n'
@@ -978,6 +1155,7 @@ printf 'BC_ENABLE_HISTORY=%s\n' "$hist"
 printf 'BC_ENABLE_EXTRA_MATH=%s\n' "$extra_math"
 printf 'BC_ENABLE_NLS=%s\n' "$nls"
 printf 'BC_ENABLE_PROMPT=%s\n' "$prompt"
+printf 'BC_ENABLE_AFL=%s\n' "$fuzz"
 printf '\n'
 printf 'BC_NUM_KARATSUBA_LEN=%s\n' "$karatsuba_len"
 printf '\n'
@@ -1012,20 +1190,45 @@ replacement='*** WARNING: Autogenerated from Makefile.in. DO NOT MODIFY ***'
 contents=$(replace "$contents" "$needle" "$replacement")
 
 if [ "$unneeded" = "" ]; then
-	contents=$(gen_file_list "$contents" "library.c")
-else
-	contents=$(gen_file_list "$contents" $unneeded)
+	unneeded="library.c"
 fi
 
+contents=$(gen_file_list "$contents" $unneeded)
+
+SRC_TARGETS=""
+
+src_files=$(find_src_files $unneeded)
+
+for f in $src_files; do
+	o=$(replace_ext "$f" "c" "o")
+	SRC_TARGETS=$(printf '%s\n\n%s: %s %s\n\t$(CC) $(CFLAGS) -o %s -c %s\n' \
+		"$SRC_TARGETS" "$o" "$headers" "$f" "$o" "$f")
+done
+
+contents=$(replace "$contents" "HEADERS" "$headers")
+
 contents=$(replace "$contents" "BC_ENABLED" "$bc")
 contents=$(replace "$contents" "DC_ENABLED" "$dc")
-contents=$(replace "$contents" "LINK" "$link")
+
+contents=$(replace "$contents" "BC_ALL_TESTS" "$bc_test")
+contents=$(replace "$contents" "BC_TESTS" "$bc_tests")
+contents=$(replace "$contents" "BC_SCRIPT_TESTS" "$bc_script_tests")
+contents=$(replace "$contents" "BC_TEST_EXEC" "$bc_test_exec")
+contents=$(replace "$contents" "TIMECONST_ALL_TESTS" "$timeconst")
+
+contents=$(replace "$contents" "DC_ALL_TESTS" "$dc_test")
+contents=$(replace "$contents" "DC_TESTS" "$dc_tests")
+contents=$(replace "$contents" "DC_SCRIPT_TESTS" "$dc_script_tests")
+contents=$(replace "$contents" "DC_TEST_EXEC" "$dc_test_exec")
 
 contents=$(replace "$contents" "LIBRARY" "$library")
 contents=$(replace "$contents" "HISTORY" "$hist")
 contents=$(replace "$contents" "EXTRA_MATH" "$extra_math")
 contents=$(replace "$contents" "NLS" "$nls")
 contents=$(replace "$contents" "PROMPT" "$prompt")
+contents=$(replace "$contents" "FUZZ" "$fuzz")
+contents=$(replace "$contents" "MEMCHECK" "$memcheck")
+
 contents=$(replace "$contents" "BC_LIB_O" "$bc_lib")
 contents=$(replace "$contents" "BC_HELP_O" "$bc_help")
 contents=$(replace "$contents" "DC_HELP_O" "$dc_help")
@@ -1057,7 +1260,18 @@ contents=$(replace "$contents" "UNINSTALL_MAN_PREREQS" "$uninstall_man_prereqs")
 contents=$(replace "$contents" "UNINSTALL_PREREQS" "$uninstall_prereqs")
 contents=$(replace "$contents" "UNINSTALL_LOCALES_PREREQS" "$uninstall_locales_prereqs")
 
+contents=$(replace "$contents" "DEFAULT_TARGET" "$default_target")
+contents=$(replace "$contents" "DEFAULT_TARGET_PREREQS" "$default_target_prereqs")
+contents=$(replace "$contents" "DEFAULT_TARGET_CMD" "$default_target_cmd")
+contents=$(replace "$contents" "SECOND_TARGET" "$second_target")
+contents=$(replace "$contents" "SECOND_TARGET_PREREQS" "$second_target_prereqs")
+contents=$(replace "$contents" "SECOND_TARGET_CMD" "$second_target_cmd")
+
 contents=$(replace "$contents" "ALL_PREREQ" "$ALL_PREREQ")
+contents=$(replace "$contents" "BC_EXEC_PREREQ" "$bc_exec_prereq")
+contents=$(replace "$contents" "BC_EXEC_CMD" "$bc_exec_cmd")
+contents=$(replace "$contents" "DC_EXEC_PREREQ" "$dc_exec_prereq")
+contents=$(replace "$contents" "DC_EXEC_CMD" "$dc_exec_cmd")
 
 contents=$(replace "$contents" "EXECUTABLES" "$executables")
 contents=$(replace "$contents" "MAIN_EXEC" "$main_exec")
@@ -1065,10 +1279,7 @@ contents=$(replace "$contents" "EXEC" "$executable")
 contents=$(replace "$contents" "TESTS" "$tests")
 
 contents=$(replace "$contents" "BC_TEST" "$bc_test")
-contents=$(replace "$contents" "BC_TIME_TEST" "$bc_time_test")
-
 contents=$(replace "$contents" "DC_TEST" "$dc_test")
-contents=$(replace "$contents" "DC_TIME_TEST" "$dc_time_test")
 
 contents=$(replace "$contents" "VG_BC_TEST" "$vg_bc_test")
 contents=$(replace "$contents" "VG_DC_TEST" "$vg_dc_test")
@@ -1086,7 +1297,17 @@ contents=$(replace "$contents" "GEN_EXEC_TARGET" "$GEN_EXEC_TARGET")
 contents=$(replace "$contents" "CLEAN_PREREQS" "$CLEAN_PREREQS")
 contents=$(replace "$contents" "GEN_EMU" "$GEN_EMU")
 
-printf '%s\n' "$contents" > "$scriptdir/Makefile"
+printf '%s\n%s\n\n' "$contents" "$SRC_TARGETS" > "$scriptdir/Makefile"
+
+if [ "$bc" -ne 0 ]; then
+	gen_tests bc BC "$extra_math" "$time_tests" $bc_test_exec
+	gen_script_tests bc "$extra_math" "$generate_tests" "$time_tests" $bc_test_exec
+fi
+
+if [ "$dc" -ne 0 ]; then
+	gen_tests dc DC "$extra_math" "$time_tests" $dc_test_exec
+	gen_script_tests dc "$extra_math" "$generate_tests" "$time_tests" $dc_test_exec
+fi
 
 cd "$scriptdir"
 
