@@ -3,7 +3,7 @@
  *
  * SPDX-License-Identifier: BSD-2-Clause
  *
- * Copyright (c) 2018-2020 Gavin D. Howard and contributors.
+ * Copyright (c) 2018-2021 Gavin D. Howard and contributors.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -89,7 +89,7 @@ static void bc_args_file(const char *file) {
 	free(buf);
 }
 
-void bc_args(int argc, char *argv[]) {
+void bc_args(int argc, char *argv[], bool exit_exprs) {
 
 	int c;
 	size_t i;
@@ -109,6 +109,7 @@ void bc_args(int argc, char *argv[]) {
 				if (vm.no_exit_exprs)
 					bc_vm_verr(BC_ERR_FATAL_OPTION, "-e (--expression)");
 				bc_args_exprs(opts.optarg);
+				vm.exit_exprs = (exit_exprs || vm.exit_exprs);
 				break;
 			}
 
@@ -119,6 +120,7 @@ void bc_args(int argc, char *argv[]) {
 					if (vm.no_exit_exprs)
 						bc_vm_verr(BC_ERR_FATAL_OPTION, "-f (--file)");
 					bc_args_file(opts.optarg);
+					vm.exit_exprs = (exit_exprs || vm.exit_exprs);
 				}
 				break;
 			}
